## Stand Alone Script for the B2B observation imaging                      
## (Customized for uid://A002/Xf06573/X20d3, R Lep Band10 B2B)           
## Script to image the B2B B7 to B10
## R Lep testing data 
## NOTE this is not an SV dataset

## Selfcal is made using 

## The Primary beam
## corrections are not applied. 

## Below 'EXTRA NOTES' sections provide some 
## more context of the steps 

### Hardcoded script Final Version
### Yoshiharu Asaki JAO & Luke Maud ESO
### For further information please contact:
### yasaki@alma.cl   JAO/NAOJ
### lmaud@eso.org   ESO
###
### If using these data please reference:
### Asaki, Y., Maud, L.T., et al. 2023 ApJS
### and B2B papers therein and published thereafter

### 28/02/2023 - CASA 6.4.1-12 version


thesteps = []
step_title = {0: 'Image the calibrators',
              1: 'Image Continuum R Lep',
              2: 'Make selfcalibration solutions for the continuum only',
              3: 'uvcontsub with mstransform for the line SPW 45',
              4: 'Image HCN maser cube',
              5: 'Selfcal using HCN maser',
              6: 'Image Continuum R Lep with selfcal from cont',
              7: 'Image Continuum R Lep with selfcal from HCN maser',
              8: 'uvcontsub with mstransform for the selfcalibrated data',
              9: 'Image the HCN maser cube with selfcal',
              10: 'Copy out fits images'
             }

if 'applyonly' not in globals(): applyonly = False
try:
  print('List of steps to be executed ...'+str(mysteps))
  thesteps = mysteps
except:
  print('global variable mysteps not set.')
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print('Executing all steps: ', thesteps)

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]
# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

import casalith

import glob

if re.search('^6.4.1', '.'.join([str(i) for i in casalith.version()[:-1]])) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 6.4.1')

mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])



  ## EXTRA NOTE - mask for point source calibrators is fixed as 
  ##              a central circle. Image parameters are also
  ##              provided to make a smaller focused image
  ##              to reduce imaging time

  cell_size_im = '0.001arcsec'
  image_size = 512
  centre_mask = ['circle[[256pix,256pix],10pix]']
  nits = 100


  # Imaging the Bandpass - only HF
  # EXTRA NOTES - as BP = DGC we want here to 
  #              select only the BP scan 
  os.system('rm -rf J0522-3627_bp.spw33_35_37_39_41_43_45_47.cont.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='J0522-3627_bp.spw33_35_37_39_41_43_45_47.cont', 
         field='J0522-3627',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         scan = '3',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)

  # DGC at HF - B10
  # EXTRA NOTE - the HF DGC is calibrated with 
  #              LF B2B transfer. Selected DGC
  #             scans specifically 
  os.system('rm -rf J0522-3627_dgc.spw33_35_37_39_41_43_45_47.cont.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='J0522-3627_dgc.spw33_35_37_39_41_43_45_47.cont', 
         field='J0522-3627',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         scan = '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # CHECK source at HF - B10
  # EXTRA NOTE - The check source is also located 'further'
  #              from the phase cal (~17deg) than the target
  #              thus at this separation angle phase 
  #              transfer is unlikes to correct the check source
  #              as it is too far away. See also Maud et al 2023 

  os.system('rm -rf J0423-0120_chk.spw33_35_37_39_41_43_45_47.cont.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='J0423-0120_chk.spw33_35_37_39_41_43_45_47.cont', 
         field='J0423-0120',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # DGC at LF - B7
  # do not need to specify scans as this field at LF is DGC only
  os.system('rm -rf J0522-3627_dgc.spw64_66.cont.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='J0522-3627_dgc.spw64_66.cont', 
         field='J0522-3627',
         interactive=False,
         spw = '64,66',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)

  os.system('rm -rf J0504-1512_ph.spw64_66.cont.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='J0504-1512_ph.spw64_66.cont', 
         field='J0504-1512',
         interactive=False,
         spw = '64,66',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  # Save first the flags as we want to reset them after
  # this process of applying the selfcal continuum
  # which WILL flag a large number of baselines 
  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
      mode = 'save',
      versionname = 'beforeContSelfCal')


  cell_size_im = '0.001arcsec'
  image_size = 512
  nits = 1000

  # in case or any rerun remove the model
  delmod(vis= 'uid___A002_Xf06573_X20d3.ms.split.cal', field='R_Lep') 

  # Target continuum
  # EXTRA NOTES - excluding some of SPW 45 as this has a HCN maser
  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual',
         field='R_Lep',
         datacolumn = 'data',
         interactive=False,
         spw = '33,35,37,39,41,43,45:0~1300;1500~1900,47', # 45 has a maser line
         deconvolver='hogbom',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0,
         savemodel='modelcolumn')


# make a selfcalibration of the continuum 
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  ## EXTRA NOTES - this will flag many longer baselines as the 
  ##               SNR is simply not high enough for the 
  ##               continuum to be used for selfcalibration
  ##               we DO NOT apply it in this step, we only
  ##               make the solutions while the model column
  ##               holds the model for RLep cont image.
  
 
  os.system('rm -rf R_Lep.p1*')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         caltable='R_Lep.p1',
         solint='inf', ## only scan based
         calmode='p',
         gaintype='T', # was G 
         spw = '33,35,37,39,41,43,45:0~1300;1500~1900,47', # not SPW 45 maser
         combine = 'spw',
         minsnr=3.0,
         refant='DV02',
         field='R_Lep')


# doing uvcontsub for SPW45 with the HCN maser
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

 
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.cal.line')
  # delete the image cont model just in case of any interferernce
  # should not be a problem as SPW45 was not imaged 
  delmod(vis= 'uid___A002_Xf06573_X20d3.ms.split.cal', field='R_Lep') 

  mstransform(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
    outputvis = 'uid___A002_Xf06573_X20d3.ms.split.cal.line',
    datacolumn = 'data',  # only a data column now 
    spw = '45',
    reindex = False,
    field= 'R_Lep',
    fitorder=1,
    fitspw='45:100~1300;1500~1850',
    niter=50,
    outframe = 'LSRK',
    regridms = True,  ## correctly grid to LSRK as the outframe
    keepflags = True)

  ## EXTRA NOTE - this method will keep HCN in SPW index 45 


# Image HCN maser cube - standard calibration
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  cell_size_im = '0.001arcsec'
  image_size = 512
  # mask made from circles to cover the main emission
  target_mask_line = ['circle[[258pix,248pix],20pix]','circle[[249pix,253pix],16pix]','circle[[247pix,241pix],16pix]','circle[[261pix,251pix],16pix]','circle[[261pix,251pix],16pix]','circle[[264pix,259pix],16pix]','circle[[251pix,261pix],16pix]','circle[[258pix,244pix],16pix]','circle[[256pix,272pix],12pix]']

  # EXTRA NOTES - cleaning is very fast as this is a small 
  #              cube for the maser. Upon testing automasking
  #              there were problems with CASA crashing which
  #              could be related to the brightness of the maser
  #              itself, and hence a manual mask is used below


  # HCN maser cube
  os.system('rm -rf R_Lep_sci.spw45.cube.I.manual-HCN.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal.line',
         datacolumn= 'data',
         imagename='R_Lep_sci.spw45.cube.I.manual-HCN',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         interactive=False,
         threshold = '0mJy',
         spw = '45',
         cell=cell_size_im, imsize=image_size, 
         niter=2000,
         specmode='cube',
         restfreq = '890.7607GHz',
         outframe = 'LSRK',
         start=1370, 
         nchan=100,
         pblimit=-1,
         width=1,
         weighting='briggs',
         robust=0.5,
         usemask = 'user',
         mask=target_mask_line )





# Selfcal using the maser
# EXTRA NOTES - the maser is very bright and we can make
#               a single channel image to use for the
#               selfcalibration
#
#               the images of HCN 1415 chan are not used
#               for science or the publication - only
#               for doing HCN maser selfcal
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  cell_size_im = '0.001arcsec'
  image_size = 512
  # mask made from circles to cover the main emission
  target_mask_line = ['circle[[258pix,248pix],20pix]','circle[[249pix,253pix],16pix]','circle[[247pix,241pix],16pix]','circle[[261pix,251pix],16pix]','circle[[261pix,251pix],16pix]','circle[[264pix,259pix],16pix]','circle[[251pix,261pix],16pix]','circle[[258pix,244pix],16pix]','circle[[256pix,272pix],12pix]']

  # EXTRA NOTES - cleaning is very fast as this is a small 
  #              cube for the maser. Upon testing automasking
  #              there were problems with CASA crashing which
  #              could be related to the brightness of the maser
  #              itself, and hence a manual mask is used below
 
  delmod(vis= 'uid___A002_Xf06573_X20d3.ms.split.cal', field='R_Lep') 

  # HCN maser self-cal (1415ch (-> V_LSRK=9.4 km/s))

  os.system('rm -rf R_Lep_sci.spw45.cube.I.manual-HCN-1415ch.*')
  tclean(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         datacolumn= 'data',
         imagename='R_Lep_sci.spw45.cube.I.manual-HCN-1415ch',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         interactive=False,
         cell=cell_size_im, imsize=image_size, 
         niter=1000,
         specmode='cube',
         restfreq = '890.7607GHz',
         outframe='LSRK',
         threshold = '0mJy',
         spw = '45',
         start=1415,
         nchan=1,
         pblimit=-1,
         weighting='briggs',
         robust=0.5,
         usemask = 'user',
         mask=target_mask_line,
         savemodel='modelcolumn')


  # EXTRA NOTES  - int selfcal as maser is very bright
  os.system('rm -rf HCN_1415ch.p1*')
  gaincal(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         caltable='HCN_1415ch.p1',
         solint='int',
         calmode='p',
         gaintype='G',
         minsnr=3.0,
         refant='DV02',
         spw='45:1415', # the single channel 
         field='R_Lep')

  # EXTRA NOTES - first apply the selfcal to 'itself' and check
  #               there are no flags during selfcal as SNR is high
  applycal(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         field = 'R_Lep',
         spw = '45',
         interp='linearPD',
         gaintable = 'HCN_1415ch.p1',
         gainfield = '')


  # now make another image and model such that we can do an AMP
  # selfcal - this is ok because we have a huge SNR on the maser
  os.system('rm -rf R_Lep_sci.spw45.cube.I.manual-HCN-1415ch-phselfcal.*')
  tclean(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         datacolumn= 'corrected',
         imagename='R_Lep_sci.spw45.cube.I.manual-HCN-1415ch-phselfcal',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         interactive=False,
         cell=cell_size_im, imsize=image_size, 
         niter=1000,
         specmode='cube',
         restfreq = '890.7607GHz',
         outframe='LSRK',
         threshold = '0mJy',
         spw = '45',
         start=1415,
         nchan=1,
         pblimit=-1,
         weighting='briggs',
         robust=0.5,
         usemask = 'user',
         mask=target_mask_line,
         savemodel='modelcolumn')


  # EXTRA NOTES  - int selfcal as maser is very bright
  os.system('rm -rf HCN_1415ch.a1*')
  gaincal(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         caltable='HCN_1415ch.a1',
         gaintable='HCN_1415ch.p1', # apply on the fly
         solint='inf', # per scan for amps 
         calmode='a',
         gaintype='G',
         minsnr=3.0,
         refant='DV02',
         spw='45:1415', # the single channel 
         field='R_Lep')


  # EXTRA NOTES - now both phase and amp applied 
  applycal(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         field = 'R_Lep',
         spw = '45',
         interp='linearPD',
         gaintable = ['HCN_1415ch.p1','HCN_1415ch.a1'],
         gainfield = '')


  # now make another image to check phase and amp selfcal 
  # applied at this stage 
  os.system('rm -rf R_Lep_sci.spw45.cube.I.manual-HCN-1415ch-phampselfcal.*')
  tclean(vis='uid___A002_Xf06573_X20d3.ms.split.cal.line',
         datacolumn= 'corrected',
         imagename='R_Lep_sci.spw45.cube.I.manual-HCN-1415ch-phampselfcal',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         interactive=False,
         cell=cell_size_im, imsize=image_size, 
         niter=1000,
         specmode='cube',
         restfreq = '890.7607GHz',
         outframe='LSRK',
         threshold = '0mJy',
         spw = '45',
         start=1415,
         nchan=1,
         pblimit=-1,
         weighting='briggs',
         robust=0.5,
         usemask = 'user',
         mask=target_mask_line,
         savemodel='modelcolumn')



# Apply of the continuum selfcal solutions to the full dataset 
# and image continuum again with SC and 'without SC' but of 
# the data column, while SC 'flags' have been added
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])


  cell_size_im = '0.001arcsec'
  image_size = 512
  nits = 1000


  # EXTRA NOTES - the flags were already saved in
  #               step 1 for first continuum image
  #               as the continuum selfcal flagged
  #               many solutions it will now flag the data
  applycal(vis='uid___A002_Xf06573_X20d3.ms.split.cal',
         field = 'R_Lep',
         spw = '33,35,37,39,41,43,45,47',
         interp='linearPD',
         gaintable = 'R_Lep.p1',
         spwmap=[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,33,36,33,38,33,40,33,42,33,44,33,46,33], # spw 33 carries the solution
         gainfield = '')

  # Target continuum
  # EXTRA NOTES - excluding some of SPW 45 as this has a HCN maser
  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-contselfcal.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-contselfcal',
         field='R_Lep',
         datacolumn = 'corrected',
         interactive=False,
         spw = '33,35,37,39,41,43,45:0~1300;1500~1900,47',
         deconvolver='hogbom',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0,
         savemodel='modelcolumn')


  # Target continuum with selfcal flags but DATA column
  # EXTRA NOTES - excluding some of SPW 45 as this has a HCN maser
  #               here we can see what was lots by the selfcal
  #               flags as we re-image the DATA column for continuum
  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-datacontselfcalflags*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-datacontselfcalflags',
         field='R_Lep',
         datacolumn = 'data',
         interactive=False,
         spw = '33,35,37,39,41,43,45:0~1300;1500~1900,47',
         deconvolver='hogbom',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0,
         savemodel='modelcolumn')


mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  cell_size_im = '0.001arcsec'
  image_size = 512
  nits = 1000

  # EXTRA NOTE - return to the initial flags, as step 6 was run
  #               the continuum selfcal flagged lots of data
  #              here we get back to the original state before
  #              running the application of the HCN maser selfcal

  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
      mode = 'restore',
      versionname = 'beforeContSelfCal')

  # APPLYCAL of the HCN maser selfcal solutions to the full dataset 

  applycal(vis='uid___A002_Xf06573_X20d3.ms.split.cal',
         field = 'R_Lep',
         spw = '33,35,37,39,41,43,45,47',
         spwmap = [[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,45,34,45,36,45,38,45,40,45,42,45,44,45,46,45],[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,45,34,45,36,45,38,45,40,45,42,45,44,45,46,45]],  # spw 45 carries the solution 
         interp=['linearPD',''],
         gaintable = ['HCN_1415ch.p1','HCN_1415ch.a1'],  
         gainfield = '')


  # Now re-image the SC continuum
  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-HCNselfcal.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-HCNselfcal',
         field='R_Lep',
         datacolumn = 'corrected',  # now the SC in corrected column
         interactive=False,
         spw = '33,35,37,39,41,43,45:0~1300;1500~1900,47', # exclude spw 45 HCN maser line
         deconvolver='hogbom',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0)


# redoing uvcontsub for SPW45 with the HCN maser after SC
# EXTRA NOTES - it is not expected that much will have changed in the 
#                continuum fitting, but it is best to redo the 
#               process after the selfcalibration as the continuum 
#               will have slightly imporved and a new fit
#               should be made
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  delmod(vis= 'uid___A002_Xf06573_X20d3.ms.split.cal', field='R_Lep') 

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.cal.SC_line')

  mstransform(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
    outputvis = 'uid___A002_Xf06573_X20d3.ms.split.cal.SC_line',
    datacolumn = 'corrected', # corrected is the SC applied 
    spw = '45',
    reindex = False,
    field= 'R_Lep',
    fitorder=1,
    fitspw='45:100~1300;1500~1850',
    niter=50,
    outframe = 'LSRK',
    regridms = True,  ## correctly grid to LSRK as the outframe
    keepflags = True)


# re do the cube with the selfcal applied then re-subtracted cont
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  cell_size_im = '0.001arcsec'
  image_size = 512
  # mask made from circles to cover the main emission
  target_mask_line = ['circle[[258pix,248pix],20pix]','circle[[249pix,253pix],16pix]','circle[[247pix,241pix],16pix]','circle[[261pix,251pix],16pix]','circle[[261pix,251pix],16pix]','circle[[264pix,259pix],16pix]','circle[[251pix,261pix],16pix]','circle[[258pix,244pix],16pix]','circle[[256pix,272pix],12pix]']

  # EXTRA NOTES - cleaning is very fast as this is a small 
  #              cube for the maser. Upon testing automasking
  #              there were problems with CASA crashing which
  #              could be related to the brightness of the maser
  #              itself, and hence a manual mask is used below


  # HCN maser cube
  os.system('rm -rf R_Lep_sci.spw45.cube.I.manual-HCN-HCNselfcal.*')
  tclean(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal.SC_line',
         datacolumn= 'data',  # the SC_line only has data, comes from corrected of split.cal
         imagename='R_Lep_sci.spw45.cube.I.manual-HCN-HCNselfcal',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         interactive=False,
         threshold = '0mJy',
         spw = '45',
         cell=cell_size_im, imsize=image_size, 
         niter=2000,
         specmode='cube',
         restfreq = '890.7607GHz',
         outframe = 'LSRK',
         start=1370, 
         nchan=100,
         pblimit=-1,
         width=1,
         weighting='briggs',
         robust=0.5,
         usemask = 'user',
         mask=target_mask_line )

 




mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  imagetofits=glob.glob('*.image*') 

  for imagefit in imagetofits: 
      exportfits(imagefit, fitsimage=imagefit+'.fits') 
