# ALMA Data Reduction Script

### This is a QA2-like script for calibration for the Band-to-Band
### testing dataset for R Lep observed at band 10 during the 2021 
### testing for the High-Frequency Long Baselines EOC campaign.
### Here, the B2B uses phase transfer from a phase calibrator 
### observed at band 7.
 
### Notes are also added 'EXTRA NOTES' during the steps below
### to help/inform inquisitive users. 

### These data were taken with SBs but specifically for
### testing purposes. These data were not, and are not science verification
### datasets and thus do not fully follow a 'standard' observational sequence.
### As the phase calibrator has some small structure steps are added
### to better correct small residual phase differences.

### For further information please contact:
### yasaki@alma.cl  JAO/NAOJ
### lmaud@eso.org   ESO 
###
### If using these data please reference:
### Asaki et al, 2023 ApJS, 
### and the B2B papers referenced therin

### The base scripts was produced by the alma script generator 
### owner and maintained by - Dirk Petry ESO
### Further modifications were made by the HF-LBC team
### Hardcoded script Final Version
### Yoshiharu Asaki JAO, Luke Maud ESO
### XX/02/2023 - CASA 6.4.1-12 version


thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs',
              9: 'Listobs, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'B2B Gain calibration Part I',
              16: 'B2B Gain calibration Part II',
              17: 'Save flags before applycal',
              18: 'Application of the bandpass and gain cal tables',
              19: 'Split out corrected column',
              20: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = False
try:
  print('List of steps to be executed ...'+str(mysteps))
  thesteps = mysteps
except:
  print('global variable mysteps not set.')
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print('Executing all steps: ', thesteps)

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]
# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

import casalith

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^6.4.1', '.'.join([str(i) for i in casalith.version()[:-1]])) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 6.4.1')


# CALIBRATE_AMPLI: 
#     field ids: ['']
# CALIBRATE_ATMOSPHERE: J0522-3627, J0504-1512, R_Lep
#     field ids: [0, 1, 3]
# CALIBRATE_BANDPASS: J0522-3627
#     field ids: [0]
# CALIBRATE_DIFFGAIN: J0522-3627
#     field ids: [0]
# CALIBRATE_FLUX: J0522-3627
#     field ids: [0]
# CALIBRATE_FOCUS: 
#     field ids: ['']
# CALIBRATE_PHASE: J0522-3627, J0504-1512
#     field ids: [0, 1]
# CALIBRATE_POINTING: J0522-3627
#     field ids: [0]
# CALIBRATE_POLARIZATION: 
#     field ids: ['']
# OBSERVE_CHECK: J0423-0120
#     field ids: [2]
# OBSERVE_TARGET: R_Lep
#     field ids: [3]


# Using reference antenna = DV02

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  if os.path.exists('uid___A002_Xf06573_X20d3.ms') == False:
    importasdm('uid___A002_Xf06573_X20d3', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xf06573_X20d3.ms.flagversions'):
      print('ERROR in importasdm. Output MS is probably not useful. Will stop here.')
      thesteps = []
  if applyonly != True: es.fixForCSV2555('uid___A002_Xf06573_X20d3.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  from casarecipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xf06573_X20d3.ms')

print("# A priori calibration")

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.listobs')
  listobs(vis = 'uid___A002_Xf06573_X20d3.ms',
    listfile = 'uid___A002_Xf06573_X20d3.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  flagdata(vis = 'uid___A002_Xf06573_X20d3.ms',
    mode = 'manual',
    spw = '5~12,17~48,53~67',
    autocorr = True,
    flagbackup = False)
  
  flagdata(vis = 'uid___A002_Xf06573_X20d3.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = False)
  
  flagcmd(vis = 'uid___A002_Xf06573_X20d3.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xf06573_X20d3.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xf06573_X20d3.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.wvr') 
  
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xf06573_X20d3.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xf06573_X20d3.ms',
    caltable = 'uid___A002_Xf06573_X20d3.ms.wvr',
    spw = [33, 35, 37, 39, 41, 43, 45, 47, 60, 62, 64, 66],
    smooth = '6.144s',
    toffset = 0,
    tie = ['R_Lep,J0504-1512', 'J0423-0120,J0504-1512'],
    statsource = 'R_Lep')
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xf06573_X20d3.ms.wvr', spw='33', antenna='DV02',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_Xf06573_X20d3.ms.wvr.plots/uid___A002_Xf06573_X20d3.ms.wvrSignalSPWs') 
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xf06573_X20d3.ms.wvr', spw='60', antenna='DV02',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_Xf06573_X20d3.ms.wvr.plots/uid___A002_Xf06573_X20d3.ms.wvrReferenceSPWs') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.tsys') 
  gencal(vis = 'uid___A002_Xf06573_X20d3.ms',
    caltable = 'uid___A002_Xf06573_X20d3.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xf06573_X20d3.ms.tsys',
    mode = 'manual',
    spw = '7:0~3;124~127,17:0~3;124~127,19:0~3;124~127,21:0~3;124~127,23:0~3;124~127,25:0~3;124~127,27:0~3;124~127,29:0~3;124~127,31:0~3;124~127,53:0~3;124~127,56:0~3;124~127,58:0~3;124~127',
    flagbackup = False)
  
  try: # protect against failure of plotbandpass in case of Tsys timestamp issues
    if applyonly != True: aU.plotbandpass(caltable='uid___A002_Xf06573_X20d3.ms.tsys', overlay='time', 
      xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
      showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, showimage=True, 
      field='', figfile='uid___A002_Xf06573_X20d3.ms.tsys.plots.overlayTime/uid___A002_Xf06573_X20d3.ms.tsys') 
  
  except:
    print('Error in plotbandpass. Skipping the time-overlay Tsys plots.')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.tsys', msName='uid___A002_Xf06573_X20d3.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  # Warning: no baseline run found for following antenna(s): ['DA41', 'DA43', 'DA49', 'DA61', 'DV08', 'DV09', 'DV10', 'DV12', 'DV14', 'DV19'].
  
  # Note: the correction for antenna DA42 is larger than 2mm.
  
  # Position for antenna DA42 is derived from baseline run made on 2021-09-13 04:39:16.
  
  # Note: the correction for antenna DA48 is larger than 2mm.
  
  # Position for antenna DA48 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Note: the correction for antenna DA53 is larger than 2mm.
  
  # Position for antenna DA53 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Note: the correction for antenna DA62 is larger than 2mm.
  
  # Position for antenna DA62 is derived from baseline run made on 2021-09-15 03:57:28.
  
  # Position for antenna DA65 is derived from baseline run made on 2021-09-13 04:39:16.
  
  # Position for antenna DV01 is derived from baseline run made on 2021-09-13 04:39:16.
  
  # Note: the correction for antenna DV02 is larger than 2mm.
  
  # Position for antenna DV02 is derived from baseline run made on 2021-09-15 03:57:28.
  
  # Note: the correction for antenna DV03 is larger than 2mm.
  
  # Position for antenna DV03 is derived from baseline run made on 2021-09-15 03:57:28.
  
  # Note: the correction for antenna DV05 is larger than 2mm.
  
  # Position for antenna DV05 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Note: the correction for antenna DV06 is larger than 2mm.
  
  # Position for antenna DV06 is derived from baseline run made on 2021-09-13 04:39:16.
  
  # Position for antenna DV07 is derived from baseline run made on 2021-09-21 04:17:41.
  
  # Note: the correction for antenna DV11 is larger than 2mm.
  
  # Position for antenna DV11 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Note: the correction for antenna DV13 is larger than 2mm.
  
  # Position for antenna DV13 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Note: the correction for antenna DV16 is larger than 2mm.
  
  # Position for antenna DV16 is derived from baseline run made on 2021-09-13 04:39:16.
  
  # Note: the correction for antenna DV22 is larger than 2mm.
  
  # Position for antenna DV22 is derived from baseline run made on 2021-09-19 03:16:48.
  
  # Position for antenna DV24 is derived from baseline run made on 2021-09-15 03:57:28.
  
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.antpos') 
  gencal(vis = 'uid___A002_Xf06573_X20d3.ms',
    caltable = 'uid___A002_Xf06573_X20d3.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA42,DA48,DA53,DA62,DA65,DV01,DV02,DV03,DV05,DV06,DV07,DV11,DV13,DV16,DV22,DV24',
  #  parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    parameter = [-1.34002e-04,-4.62800e-03,-1.15900e-03,-2.43700e-03,3.39600e-03,-7.36002e-04,-2.64000e-03,3.60800e-03,-8.52003e-04,2.08000e-03,-2.54999e-04,3.89001e-04,-6.50000e-04,1.69900e-03,-1.43001e-04,8.69000e-04,-9.07000e-04,-2.27999e-04,4.37001e-04,2.96900e-03,4.18999e-04,1.88800e-03,-1.38400e-03,-5.31999e-04,2.59800e-03,9.91002e-04,-1.29500e-03,1.13000e-04,-2.18100e-03,2.87001e-04,-1.04700e-03,6.65000e-04,7.09994e-05,-3.10400e-03,4.59200e-03,3.39979e-05,2.65800e-03,-7.72500e-03,-1.73500e-03,1.66500e-03,-4.60300e-03,-1.96998e-04,2.63200e-03,1.42600e-03,-5.54000e-04,-1.11800e-03,5.15000e-04,1.55000e-04])
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  # DV13     2.65800e-03   -7.72500e-03   -1.73500e-03    8.35170e-03      2021-09-19 03:16:48
  # DV11    -3.10400e-03    4.59200e-03    3.39979e-05    5.54278e-03      2021-09-19 03:16:48
  # DV16     1.66500e-03   -4.60300e-03   -1.96998e-04    4.89884e-03      2021-09-13 04:39:16
  # DA42    -1.34002e-04   -4.62800e-03   -1.15900e-03    4.77280e-03      2021-09-13 04:39:16
  # DA53    -2.64000e-03    3.60800e-03   -8.52003e-04    4.55117e-03      2021-09-19 03:16:48
  # DA48    -2.43700e-03    3.39600e-03   -7.36002e-04    4.24423e-03      2021-09-19 03:16:48
  # DV05     2.59800e-03    9.91002e-04   -1.29500e-03    3.06737e-03      2021-09-19 03:16:48
  # DV22     2.63200e-03    1.42600e-03   -5.54000e-04    3.04431e-03      2021-09-19 03:16:48
  # DV02     4.37001e-04    2.96900e-03    4.18999e-04    3.03010e-03      2021-09-15 03:57:28
  # DV03     1.88800e-03   -1.38400e-03   -5.31999e-04    2.40063e-03      2021-09-15 03:57:28
  # DV06     1.13000e-04   -2.18100e-03    2.87001e-04    2.20270e-03      2021-09-13 04:39:16
  # DA62     2.08000e-03   -2.54999e-04    3.89001e-04    2.13137e-03      2021-09-15 03:57:28
  # DA65    -6.50000e-04    1.69900e-03   -1.43001e-04    1.82471e-03      2021-09-13 04:39:16
  # DV01     8.69000e-04   -9.07000e-04   -2.27999e-04    1.27663e-03      2021-09-13 04:39:16
  # DV07    -1.04700e-03    6.65000e-04    7.09994e-05    1.24237e-03      2021-09-21 04:17:41
  # DV24    -1.11800e-03    5.15000e-04    1.55000e-04    1.24064e-03      2021-09-15 03:57:28
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])
  
  from casarecipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xf06573_X20d3.ms', tsystable = 'uid___A002_Xf06573_X20d3.ms.tsys', tsysChanTol = 1)
  
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms',
    field = '0',
    spw = '33,35,37,39,41,43,45,47,60,62,64,66',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.tsys', 'uid___A002_Xf06573_X20d3.ms.wvr', 'uid___A002_Xf06573_X20d3.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = ['','nearest, linear',''],
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms',
    field = '1',
    spw = '60,62,64,66',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.tsys', 'uid___A002_Xf06573_X20d3.ms.wvr', 'uid___A002_Xf06573_X20d3.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = ['','nearest, linear',''],
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  # Note: J0423-0120 didn't have any Tsys measurement, so I used the one made on R_Lep. This is probably Ok.
  
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms',
    field = '2',
    spw = '33,35,37,39,41,43,45,47',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.tsys', 'uid___A002_Xf06573_X20d3.ms.wvr', 'uid___A002_Xf06573_X20d3.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = ['','nearest, linear',''],
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms',
    field = '3',
    spw = '33,35,37,39,41,43,45,47',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.tsys', 'uid___A002_Xf06573_X20d3.ms.wvr', 'uid___A002_Xf06573_X20d3.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = ['','nearest, linear',''],
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_Xf06573_X20d3.ms') 
  

# Split out science SPWs
# Low frequency Spws 60 and 62 are not available because the opposite wrong sideband were selected in recording the data.
#
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split') 
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.flagversions') 
  
  ## EXTRA NOTE - due to some issues with the recording of the B2B 
  ##              in Walsh switching mode at B10, the 
  ##              low frequency data, SPW 60 and 62 do 
  ##              not contain useful information

  mstransform(vis = 'uid___A002_Xf06573_X20d3.ms',
    outputvis = 'uid___A002_Xf06573_X20d3.ms.split',
    datacolumn = 'corrected',
    spw = '33,35,37,39,41,43,45,47,64,66',
    reindex = False,
    keepflags = True)
  

# Listobs, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.listobs')
  listobs(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    listfile = 'uid___A002_Xf06573_X20d3.ms.split.listobs')
  
  if not os.path.exists('uid___A002_Xf06573_X20d3.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split',
      mode = 'save',
      versionname = 'Original')
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    mode = 'shadow',
    flagbackup = False)


# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  setjy(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    standard = 'manual',
    field = 'J0522-3627',
    spw = '33,35,37,39,41,43,45,47,64,66',
    usescratch = True,
    fluxdensity = [4.1374448999766775, 0, 0, 0],
    spix = -0.2500326142272155,
    reffreq = '896.2293273244093GHz')
  
  # fluxDensityUncertainty = 0.14325968401037414
  # meanAge = 0.0


  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])
  
  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass') 
  
  ## EXTRA NOTES - use the BP explicit scan at the HF and use
  ##              the various DGC scans for the LF
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass',
    field = '0', # J0522-3627
    spw = '33:0~1919,35:0~1919,37:0~1919,39:0~1919,41:0~1919,43:0~1919,45:0~1919,47:0~1919,64:0~1919,66:0~1919',
    scan = '3,6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    solint = 'int',
    refant = 'DV02',
    refantmode = 'strict',
    minsnr = 3.0,
    calmode = 'p')

  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False) 
  
  # HF bandpass
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass',
    field = '0', # J0522-3627
    scan = '3',
    solint = 'inf',
    minsnr = 3.0,
    combine = 'scan',
    refant = 'DV02',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass')

  # LF bandpass
  bandpass(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass',
    field = '0', # J0522-3627
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    solint = 'inf',
    minsnr = 3.0,
    combine = 'scan',
    refant = 'DV02',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass',
    append = True)
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.bandpass', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch') 
  
  # HF bandpass smooth to 20 ch for SNR
  bandpass(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch',
    field = '0', # J0522-3627
    scan = '3',
    solint = 'inf,20ch',
    combine = 'scan',
    refant = 'DV02',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass')
  
  # LF bandpass smooth to 20ch for SNR
  bandpass(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch',
    field = '0', # J0522-3627
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    solint = 'inf,20ch',
    minsnr = 3.0,
    combine = 'scan',
    refant = 'DV02',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.ap_pre_bandpass',
    append = True)
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  

# B2B Gain calibration Part I
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  ## EXTRA NOTES - refantmode is strict as we do not want the 
  ##               refant to change during the B2B process
  ##               where we transfer between bands 

  # do a phase offset for LF so all SPW can be merged for better SNR
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff',
    field = '0',
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    spw = '64,66',
    solint = 'inf',
    refant = 'DV02',
    gaintype = 'G',
    calmode = 'p',
    combine = 'scan',
    minsnr = 3.0,
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  # LF temporal solutions fast
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_int')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_int',
    field = '0',
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    spw = '64,66',
    solint = 'int',
    refant = 'DV02',
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    combine = 'spw', 
    refantmode = 'strict',
    interp = ['linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch', 'uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff'],
    gainfield = ['',''])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_int', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  # LF temporal solutions slow
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf',
    field = '0',
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    spw = '64,66',
    solint = 'inf',
    refant = 'DV02',
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    combine = 'spw',
    refantmode = 'strict',
    interp = ['linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff'],
    gainfield = ['',''])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  ## HF fast solutions
  ## EXTRA NOTES - these are just for checking purposes
  ##               these look rather messy as it is difficult to
  ##               see by eye over the small periods of time how the 
  ##               phases are tracking the atmosphere variability
  ##               it is better to later check residual plots 
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCphasehigh_int')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCphasehigh_int',
    field = '0',
    scan= '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
    spw = '33,35,37,39,41,43,45,47',
    solint = 'int',
    refant = 'DV02',
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    refantmode = 'strict',
    interp = ['linear','linear'],
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch',
    gainfield = [''])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCphasehigh_int', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  # get the DGC band-offset
  # apply the LF to the HF and find the offset for each DGC 'group'
  # if there are multiple DGC sequences - this does each one independently (hence 'multi')
  # EXTRA NOTES - spw map applies LF from SPW 64 to the HF SPWs 
  #               the plots show this is relatively constant within a few tens of degrees or
  #               less per DGC 'block' - very good for B10 B2B 
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi',
    field = '0',
    scan = '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
    spw = '33,35,37,39,41,43,45,47',
    solint = '8min',  # max. duration of one DGC visit block
    refant = 'DV02',
    refantmode = 'strict',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='scan',
    interp=['linear','linearPD'],
    spwmap=[[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64]],
    gainfield = ['',''],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  

  ## EXTRA NOTES - these plots only run if global variable applyone = False
  ##               generally these are for data analysts to confirm the 
  ##               quality of the calibration residual

  if applyonly != True:
    # QA2 check to verify the slow (inf) LF to HF and offset worked
    # apply the LF to the HF and the multi offset
    # EXTRA NOTES - spw map applies LF from SPW 64 to the HF SPWs 
    #              the plots are centered on zero and generally are within +/-50deg
    #              this is good for B10 residuals (in the DGC) after B2B transfer
    os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi')
    gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
      caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi',
      field = '0',
      scan = '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
      spw = '33,35,37,39,41,43,45,47',
      solint = 'inf',
      refant = 'DV02',
      refantmode = 'strict',
      gaintype = 'G',
      calmode = 'p',
      minsnr=3.0,
      interp=['linear','linearPD','linear'],
      spwmap=[[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64],[]],
      gainfield = ['','',''],
      gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf','uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  if applyonly != True:
    # QA2 check to verify the fast (int) LF to HF and offset worked
    # EXTRA NOTES - spw map applies LF from SPW 64 to the HF SPWs 
    #              the plots are centered on zero and generally are within +/-50deg
    #              this is good for B10 residuals (in the DGC) after B2B transfer
    os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi_int')
    gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
      caltable = 'uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi_int',
      field = '0',
      scan = '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
      spw = '33,35,37,39,41,43,45,47',
      solint = 'int',
      refant = 'DV02',
      refantmode = 'strict',
      gaintype = 'G',
      calmode = 'p',
      minsnr=3.0,
      interp=['linear','linearPD','linear'],
      spwmap=[[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64],[]],
      gainfield = ['','',''],
      gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_inf','uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.DGCresidual_offset_multi_int', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  

# B2B Gain calibration Part II
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  # LF fast phase as a QA2 check for stability scan drop-outs
  # EXTRA NOTE - checking the plots shows no immediate outliers or jumps
  #             noting that there will be a slight phase offset between 
  #             solutions of the phase calibrator and DGC which are both
  #             plotted
  #             DV05, DV13, DV18 and DV21 look 'noisy' but these are the 
  #             the longest baselines in the configuration and the long
  #             term atmospheric variations will be the largest
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.phaselow_int')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.phaselow_int',
    field = '0,1',
    spw = '64,66',
    solint = 'int',
    refant = 'DV02',
    refantmode = 'strict',
    combine='spw', 
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.phaselow_int', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  # LF slow phase solution
  # EXTRA NOTE - these will be applied to the target and check source
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.phaselow_inf')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.phaselow_inf',
    field = '0,1',
    spw = '64,66',
    solint = 'inf',
    refant = 'DV02',
    refantmode='strict',
    combine='spw', 
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.phaselow_inf', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  ## HF phase up on all strong sources, i.e. the BP, flux cal, and DGC for amp gain table
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.phasehigh_int')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.phasehigh_int',
    field = '0',
    spw = '33,35,37,39,41,43,45,47',
    solint = 'int',
    refant = 'DV02',
    refantmode = 'strict',
    gaintype = 'G',
    calmode = 'p',
    minsnr = 3.0,
    gaintable = 'uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch')
 
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.phasehigh_int', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  
  # Amplitude calibration HF
  # EXTRA NOTES - solint is 5min to cover each DGC block for the  
  #               amplitude calibration as the BP = DGC in these data
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.ampli_inf')
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.ampli_inf',
    field = '0',
    scan = '3,7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
    spw = '33,35,37,39,41,43,45,47',
    solint = '5min',
    combine = 'scan',
    refant = 'DV02',
    gaintype = 'T',
    calmode = 'a',
    minsnr = 3.0,
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.phasehigh_int'],
    gainfield = ['0',''])
  
  # Amplitude calibration LF, append
  # EXTRA NOTE - the LF is calibrated with phases from SPW 64
  #              linearPD is also used to ensure correct phase
  #              scaling to SPW 66
  gaincal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    caltable = 'uid___A002_Xf06573_X20d3.ms.split.ampli_inf',
    field = '0',
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    spw = '64,66',
    solint = '5min',
    combine = 'scan',
    refant = 'DV02',
    gaintype = 'T',
    calmode = 'a',
    append = True,
    minsnr = 3.0,
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff','uid___A002_Xf06573_X20d3.ms.split.phaselow_int'],
    interp=['linear','linear','linearPD'],
    spwmap = [[], [], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 64, 61, 64, 63, 64, 65, 64]],
    gainfield = ['0','',''])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xf06573_X20d3.ms.split.ampli_inf', msName='uid___A002_Xf06573_X20d3.ms.split', interactive=False)
  

# Save flags before applycal
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])
  
  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  

# Application of the bandpass and gain cal tables
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  # calibrating the bandpass/fluxcalibrator
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '0',
    spw = '33,35,37,39,41,43,45,47',
    scan = '3',
    interp=['linear','linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.phasehigh_int','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    gainfield = ['0', '0', '0'])
  
  # calibrating the DGC with the LF phases
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '0',
    spw = '33,35,37,39,41,43,45,47',
    scan = '7,9,11,13,15,74,76,78,80,82,141,143,145,147,149,180,182,184,186,188',
    interp=['linear','linearPD','linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.phaselow_inf','uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    spwmap = [[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64],[],[]],
    gainfield = ['0','0','0','0'])
  
  ## calibrating the target
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '3',
    spw = '33,35,37,39,41,43,45,47',
    interp = ['linear','linearPD', 'linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.phaselow_inf','uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    spwmap = [[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64],[],[]],
    gainfield = ['0', '1','0','0'])
  
  ## calibrating the checksource
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '2',
    spw = '33,35,37,39,41,43,45,47',
    interp = ['linear','linearPD','linear','linear'],
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.phaselow_inf','uid___A002_Xf06573_X20d3.ms.split.DGCoffset_multi','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    spwmap = [[],[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64],[],[]],
    gainfield = ['0','1','0','0'])
 
  ## calibration of the LF data
  
  # calibrating the DGC LF
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '0',
    spw = '64,66',
    scan = '6,8,10,12,14,16,73,75,77,79,81,83,140,142,144,146,148,150,179,181,183,185,187,189',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff','uid___A002_Xf06573_X20d3.ms.split.phaselow_inf','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    interp = ['linear','linear','linearPD','linear'],
    spwmap = [[], [], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 64, 61, 64, 63, 64, 65, 64],[]],
    gainfield = ['0','','0','0'])
  
  # calibrating the phasecal LF
  applycal(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    field = '1',
    spw = '64,66',
    gaintable = ['uid___A002_Xf06573_X20d3.ms.split.bandpass_smooth20ch','uid___A002_Xf06573_X20d3.ms.split.DGCphaselow_phasediff','uid___A002_Xf06573_X20d3.ms.split.phaselow_inf','uid___A002_Xf06573_X20d3.ms.split.ampli_inf'],
    interp = ['linear','linear','linearPD','linear'],
    spwmap = [[], [], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 64, 34, 64, 36, 64, 38, 64, 40, 64, 42, 64, 44, 64, 46, 64, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 64, 61, 64, 63, 64, 65, 64],[]],
    gainfield = ['0','','1','0'])
  

# Split out corrected column
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',
   'CALIBRATE_DIFFGAIN#REFERENCE',
   'CALIBRATE_DIFFGAIN#SIGNAL',
   'CALIBRATE_FLUX#ON_SOURCE',
   'CALIBRATE_PHASE#ON_SOURCE',
   'CALIBRATE_PHASE#REFERENCE',
   'CALIBRATE_WVR#AMBIENT',
   'CALIBRATE_WVR#HOT',
   'CALIBRATE_WVR#OFF_SOURCE',
   'CALIBRATE_WVR#ON_SOURCE',
   'CALIBRATE_WVR#REFERENCE',
   'CALIBRATE_WVR#SIGNAL',
   'OBSERVE_CHECK_SOURCE#ON_SOURCE',
   'OBSERVE_TARGET#ON_SOURCE']
  
  ## Continumm process
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.cal') 
  os.system('rm -rf uid___A002_Xf06573_X20d3.ms.split.cal.flagversions') 
  
  # EXTRA NOTE - chanaverage and chanbins can be provided
  #              to make a slightly smaller final MS
  #              and potentially speed up imaging
  mstransform(vis = 'uid___A002_Xf06573_X20d3.ms.split',
    outputvis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
    datacolumn = 'corrected',
    reindex = False,
    intent = ','.join(listOfIntents),
    #chanaverage=True, chanbin=32,
    keepflags = True)

  # EXTRA NOTE -  HCN maser line in SPW 45 channels 42~44
  #            science SPWs are '33,35,37,39,41,43,45,47'

# Save flags after applycal
mystep = 20
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  flagmanager(vis = 'uid___A002_Xf06573_X20d3.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
  
